@@ -16,13 +16,14 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#ident "$Id: utf8echo.c,v 1.9 2002/10/17 16:17:40 nalin Exp $"
+#ident "$Id: utf8echo.c,v 1.10 2002/12/03 03:49:59 nalin Exp $"
 #include "../config.h"
 #include <glib.h>
 #include <limits.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
+#include <unistd.h>
 #include "table.h"
 #define ESC ""
 
@@ -35,10 +36,12 @@ main(int argc, char **argv)
 	wchar_t w;
 	char *inbuf, *outbuf, *p;
 	size_t insize, outsize;
-	gboolean dont_reset = FALSE;
+	gboolean do_reset = FALSE;
 
 	if (argc < 2) {
 		printf("usage: [-r] %s index [...]\n", argv[0]);
+		printf("        -r  reset to default terminal encoding "
+		       "when finished\n");
 		return 1;
 	}
 
@@ -47,10 +50,12 @@ main(int argc, char **argv)
 		return 1;
 	}
 
-	printf(ESC "%%G");
+	if (isatty(STDOUT_FILENO)) {
+		printf(ESC "%%G");
+	}
 	for (i = 1; i < argc; i++) {
 		if (strcmp(argv[i], "-r") == 0) {
-			dont_reset = !dont_reset;
+			do_reset = !do_reset;
 			continue;
 		}
 		w = (wchar_t)strtol(argv[i], &p, 0);
@@ -63,7 +68,7 @@ main(int argc, char **argv)
 			printf("%*s", outbuf - buf, buf);
 		}
 	}
-	if (!dont_reset) {
+	if (isatty(STDOUT_FILENO) && do_reset) {
 		printf(ESC "%%@\n");
 	}
 
